@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
-// SPDX-FileCopyrightText: 2022–2024 Jan Engelhardt
+// SPDX-FileCopyrightText: 2022–2025 Jan Engelhardt
 #include <algorithm>
 #include <functional>
 #include <cctype>
@@ -101,13 +101,14 @@ static constexpr srgb888 win_palette[] = {
 	{0x00,0x00,0xff}, {0xff,0x00,0xff}, {0x00,0xff,0xff}, {0xff,0xff,0xff},
 };
 
-static unsigned int xterm_fg, xterm_bg, xterm_bd, g_verbose;
+static unsigned int xterm_fg, xterm_bg, xterm_bd, g_verbose = 1;
 static double g_continuous_gamma;
 static const Babl *lch_space, *srgb_space, *srgb888_space;
 static Eigen::Matrix3d xyz_to_lrgb_matrix;
 
 static constexpr HXoption g_options_table[] = {
-	{{}, 'v', HXTYPE_NONE, &g_verbose, {}, {}, {}, "Debugging"},
+	{{}, 'q', HXTYPE_NONE | HXOPT_DEC, &g_verbose, {}, {}, {}, "Reduce noise"},
+	{{}, 'v', HXTYPE_NONE | HXOPT_INC, &g_verbose, {}, {}, {}, "Debugging"},
 	HXOPT_AUTOHELP,
 	HXOPT_TABLEEND,
 };
@@ -207,7 +208,12 @@ static hsl parse_hsl(const char *str)
 {
 	hsl c;
 	if (*str != '#') {
-		sscanf(str, "%lf,%lf,%lf", &c.h, &c.s, &c.l);
+		double h, s, l;
+		if (sscanf(str, "%lf,%lf,%lf", &h, &s, &l) != 3) {
+			fprintf(stderr, "Illegal HSL input: \"%s\"\n", str);
+			return c;
+		}
+		c = {h, s, l};
 		return c;
 	}
 	srgb888 r;
@@ -304,15 +310,19 @@ static std::vector<lch> lchtint(const lch &base, const std::vector<lch> &light)
 
 static void colortable_256()
 {
+	char v = g_verbose >= 1 ? '.' : ' ';
 	for (unsigned int b = 0; b < 256; b += 32) {
 		for (unsigned int g = 0; g < 256; g += 32) {
 			for (unsigned int r = 0; r < 256; r += 16)
-				printf("\e[30;48;2;%u;%u;%um.", r, g, b);
+				printf("\e[30;48;2;%u;%u;%um%c", r, g, b, v);
 			printf("\e[0m\n");
 		}
 	}
 	for (unsigned int c = 0x0; c <= 0xFF; ++c) {
-		printf("\e[30;48;5;%um-%02x-", c, c);
+		if (g_verbose >= 1)
+			printf("\e[30;48;5;%um-%02x-", c, c);
+		else
+			printf("\e[30;48;5;%um  ", c);
 		if ((c - 3) % 6 == 0)
 			printf("\e[0m\n");
 	}
@@ -324,7 +334,10 @@ static void colortable_16(std::function<void(int, int, int)> pr = nullptr)
 	if (pr == nullptr) {
 		printf("                  ┌─ bright ───────┐┌─ bold ─────────┐┌─ reverse ──────┐\n");
 		pr = [](int bg, int fg, int sp) {
-			printf("%x%c", bg >= 0 ? bg : 0, fg < 10 ? '0' + fg : 'a' + fg - 10);
+			if (g_verbose == 0)
+				printf("  ");
+			else
+				printf("%x%c", bg >= 0 ? bg : 0, fg < 10 ? '0' + fg : 'a' + fg - 10);
 		};
 		modes = {0, 90, 1, 7};
 	}
@@ -384,8 +397,8 @@ void palstat::compute_sums(unsigned int xlim, unsigned int ylim, gvstat &gs)
 				gs.adj_sum += delta[y][x];
 		}
 	}
-	gs.avg = gs.sum / gs.pairs;
-	gs.adj_avg = gs.adj_sum / (gs.pairs - gs.penalized);
+	gs.avg = gs.pairs > 0 ? gs.sum / gs.pairs : 0;
+	gs.adj_avg = gs.pairs - gs.penalized > 0 ? gs.adj_sum / (gs.pairs - gs.penalized) : 0;
 }
 
 void palstat::compute_sums()
@@ -534,10 +547,14 @@ static void cx_report(const palstat &o)
 	cx_report(o.x88, " 8x8 ");
 }
 
-static void cxl_command(const std::vector<lch> &lch_pal)
+static void cxl_command(const std::vector<lch> &lch_pal) try
 {
-	printf("\e[1m════ Difference of the L components ════\e[0m\n");
+	if (lch_pal.size() < 16) {
+		fprintf(stderr, "cxl_compute: LCh palette must have 16 entries\n");
+		return;
+	}
 	auto sb = cxl_compute(lch_pal);
+	printf("\e[1m════ Difference of the L components ════\e[0m\n");
 	colortable_16([&](int bg, int fg, int special) {
 		if (special || fg >= 16 || bg >= 16 || fg == bg)
 			printf("   ");
@@ -545,10 +562,15 @@ static void cxl_command(const std::vector<lch> &lch_pal)
 			printf("%3.0f", sb.delta[bg][fg]);
 	});
 	cx_report(sb);
+} catch (int) {
 }
 
 static void cxa_command(const std::vector<srgb888> &pal)
 {
+	if (pal.size() < 16) {
+		fprintf(stderr, "cxl_compute: RGB palette must have 16 entries\n");
+		return;
+	}
 	printf("\e[1m════ APCA lightness contrast ════\e[0m\n");
 	auto sb = cxa_compute(pal);
 	colortable_16([&](int bg, int fg, int special) {
@@ -570,7 +592,7 @@ static std::vector<lch> equalize(std::vector<lch> la, unsigned int sbl_size,
 	std::sort(sbl.begin(), sbl.end(),
 		[&](unsigned int x, unsigned int y) { return la[x].l < la[y].l; });
 
-	if (g_verbose) {
+	if (g_verbose >= 2) {
 		fprintf(stderr, "equalize(%zu) in: ", sbl.size());
 		for (auto z : sbl)
 			fprintf(stderr, "%f(\e[%u;3%um%x\e[0m) ", la[z].l, !!(z & 0x8), z & 0x7, z);
@@ -579,10 +601,10 @@ static std::vector<lch> equalize(std::vector<lch> la, unsigned int sbl_size,
 	for (unsigned int idx = 1; idx < sbl.size(); ++idx) {
 		unsigned int z = sbl[idx];
 		la[z].l = (gray - blue) * (idx - 1) / (sbl.size() - 2) + blue + la[sbl[0]].l;
-		if (g_verbose)
+		if (g_verbose >= 2)
 			fprintf(stderr, "%f(\e[%u;3%um%x\e[0m) ", la[z].l, !!(z & 0x8), z & 0x7, z);
 	}
-	if (g_verbose)
+	if (g_verbose >= 2)
 		fprintf(stderr, "\n");
 	return la;
 }
@@ -796,7 +818,7 @@ static double eval_rd(mpalette &mpal, size_t idx, char reg)
 	case 'x': return mpal.x;
 	case 'y': return mpal.y;
 	case 'z': return mpal.z;
-	default: throw -1;
+	default: throw "Illegal register";
 	}
 }
 
@@ -831,6 +853,7 @@ static token_entry eval_grp(const token_vector &tokens, mpalette &mpal, size_t i
 		return {};
 	} else if (tokens[1].type != token_type::op) {
 		eval_help("Expected middle token to be an operator", tokens);
+		return {};
 	}
 
 	auto op = std::get<char>(tokens[1].val);
@@ -881,16 +904,19 @@ static token_entry eval_grp(const token_vector &tokens, mpalette &mpal, size_t i
 	return lhs;
 }
 
-static int do_eval(const char *cmd, mpalette &mpal, const std::vector<size_t> &indices = {})
+static int do_eval(const char *cmd, mpalette &mpal,
+    const std::vector<size_t> &indices = {}) try
 {
 	token_vector tokens;
 	auto ret = eval_tokenize(cmd, nullptr, tokens);
 	if (ret != 0)
 		return ret;
-	if (g_verbose)
+	if (g_verbose >= 2)
 		fprintf(stderr, "# expr parsed as: %s\n", repr(tokens).c_str());
-	if (mpal.la.size() != mpal.ra.size())
-		throw "Programming error";
+	if (mpal.la.size() != mpal.ra.size()) {
+		fprintf(stderr, "Programming error / Debug me here\n");
+		return -1;
+	}
 	if (indices.empty()) {
 		for (size_t i = 0; i < mpal.la.size(); ++i) {
 			auto d = eval_grp(tokens, mpal, i);
@@ -907,6 +933,12 @@ static int do_eval(const char *cmd, mpalette &mpal, const std::vector<size_t> &i
 		}
 	}
 	return 0;
+} catch (const std::bad_variant_access &e) {
+	fprintf(stderr, "%s\n", e.what());
+	return -1;
+} catch (const char *e) {
+	fprintf(stderr, "%s\n", e);
+	return -1;
 }
 
 std::vector<size_t> parse_range(const char *s_input)
@@ -952,9 +984,10 @@ int main(int argc, char **argv)
 		~bb_guard() { ::babl_exit(); }
 	};
 	bb_guard bbg;
+	HXopt6_auto_result argp;
 
-	if (HX_getopt5(g_options_table, argv, &argc, &argv,
-	    HXOPT_RQ_ORDER | HXOPT_USAGEONERR) != HXOPT_ERR_SUCCESS)
+	if (HX_getopt6(g_options_table, argc, argv, &argp,
+	    HXOPT_RQ_ORDER | HXOPT_USAGEONERR | HXOPT_ITER_ARGS) != HXOPT_ERR_SUCCESS)
 		return EXIT_FAILURE;
 
 	xyz_to_lrgb_matrix = make_lrgb_matrix(to_xyz(illuminant_d(6500)));
@@ -974,31 +1007,32 @@ int main(int argc, char **argv)
 		return EXIT_FAILURE;
 	}
 
-	while (*++argv != nullptr) {
-		auto ptr = strchr(*argv, '=');
+	for (int ni = 0; ni < argp.nargs; ++ni) {
+		auto le_arg = argp.uarg[ni];
+		auto ptr = strchr(le_arg, '=');
 		auto arg1 = ptr != nullptr ? strtod(ptr + 1, nullptr) : 0;
 		bool mod_ra = false, mod_la = false;
 
-		if (strcmp(*argv, "vga") == 0) {
+		if (strcmp(le_arg, "vga") == 0) {
 			mpal.ra = {std::begin(vga_palette), std::end(vga_palette)};
 			mod_ra = true;
-		} else if (strcmp(*argv, "vgs") == 0) {
+		} else if (strcmp(le_arg, "vgs") == 0) {
 			mpal.ra = {std::begin(vgasat_palette), std::end(vgasat_palette)};
 			mod_ra = true;
-		} else if (strcmp(*argv, "win") == 0) {
+		} else if (strcmp(le_arg, "win") == 0) {
 			mpal.ra = {std::begin(win_palette), std::end(win_palette)};
 			mod_ra = true;
-		} else if (strncmp(*argv, "loadpal=", 8) == 0) {
-			if (loadpal(&argv[0][8], mpal.ra) != 0)
+		} else if (strncmp(le_arg, "loadpal=", 8) == 0) {
+			if (loadpal(&le_arg[8], mpal.ra) != 0)
 				return EXIT_FAILURE;
 			mod_ra = true;
-		} else if (strncmp(*argv, "loadreg=", 8) == 0) {
-			mpal = allpal[&argv[0][8]];
-		} else if (strncmp(*argv, "savereg=", 8) == 0) {
-			allpal[&argv[0][8]] = mpal;
-		} else if (strncmp(*argv, "blend=", 6) == 0) {
+		} else if (strncmp(le_arg, "loadreg=", 8) == 0) {
+			mpal = allpal[&le_arg[8]];
+		} else if (strncmp(le_arg, "savereg=", 8) == 0) {
+			allpal[&le_arg[8]] = mpal;
+		} else if (strncmp(le_arg, "blend=", 6) == 0) {
 			char *end = nullptr;
-			auto pct = strtod(&argv[0][6], &end);
+			auto pct = strtod(&le_arg[6], &end);
 			if (*end == ',') {
 				++end;
 				auto bi = allpal.find(end);
@@ -1010,21 +1044,21 @@ int main(int argc, char **argv)
 				}
 			}
 
-		} else if (strncmp(*argv, "eval@", 5) == 0) {
-			auto eqsign = strchr(&argv[0][5], '=');
+		} else if (strncmp(le_arg, "eval@", 5) == 0) {
+			auto eqsign = strchr(&le_arg[5], '=');
 			if (eqsign != nullptr) {
 				*eqsign++ = '\0';
-				auto indices = parse_range(&argv[0][5]);
+				auto indices = parse_range(&le_arg[5]);
 				if (do_eval(eqsign, mpal, indices) != 0)
-					break;
+					return EXIT_FAILURE;
 			}
-		} else if (strncmp(*argv, "eval=", 5) == 0) {
-			if (do_eval(&argv[0][5], mpal) != 0)
-				break;
-		} else if (**argv == '(' || (strchr(EVAL_REGS, **argv) && argv[0][1] == '=')) {
-			if (do_eval(argv[0], mpal) != 0)
-				break;
-		} else if (strncmp(*argv, "ild=", 4) == 0) {
+		} else if (strncmp(le_arg, "eval=", 5) == 0) {
+			if (do_eval(&le_arg[5], mpal) != 0)
+				return EXIT_FAILURE;
+		} else if (*le_arg == '(' || (strchr(EVAL_REGS, *le_arg) && le_arg[1] == '=')) {
+			if (do_eval(le_arg, mpal) != 0)
+				return EXIT_FAILURE;
+		} else if (strncmp(le_arg, "ild=", 4) == 0) {
 			fprintf(stderr, "New white_point D_%.2f:\n", arg1 / 100);
 			auto a = illuminant_d(arg1);
 			fprintf(stderr, "{x=%.15f, y=%.15f}\n", a.x, a.y);
@@ -1034,7 +1068,7 @@ int main(int argc, char **argv)
 			std::stringstream ss;
 			ss << xyz_to_lrgb_matrix;
 			fprintf(stderr, "XYZ-to-LRGB matrix:\n%s\n", ss.str().c_str());
-		} else if (strcmp(*argv, "lch") == 0) {
+		} else if (strcmp(le_arg, "lch") == 0) {
 			printf("#L,c,h\n");
 			unsigned int cnt = 0;
 			for (auto &e : mpal.la) {
@@ -1043,30 +1077,30 @@ int main(int argc, char **argv)
 					cnt, e.l, e.c, e.h);
 				++cnt;
 			}
-		} else if (strncmp(*argv, "hsltint=", 8) == 0) {
-			mpal.ra = hsltint(parse_hsl(&argv[0][8]), mpal.la);
+		} else if (strncmp(le_arg, "hsltint=", 8) == 0) {
+			mpal.ra = hsltint(parse_hsl(&le_arg[8]), mpal.la);
 			mod_ra = true;
-		} else if (strncmp(*argv, "lchtint=", 8) == 0) {
-			auto base = parse_hsl(&argv[0][8]);
+		} else if (strncmp(le_arg, "lchtint=", 8) == 0) {
+			auto base = parse_hsl(&le_arg[8]);
 			auto v = to_lch(to_srgb(base));
-			if (g_verbose)
-				fprintf(stderr, "# converted %s to LCh(%f,%f,%f)\n", &argv[0][8], v.l, v.c, v.h);
+			if (g_verbose >= 2)
+				fprintf(stderr, "# converted %s to LCh(%f,%f,%f)\n", &le_arg[8], v.l, v.c, v.h);
 			mpal.la = lchtint(v, mpal.la);
 			mod_la = true;
-		} else if (strcmp(*argv, "emit") == 0 || strcmp(*argv, "xfce") == 0) {
+		} else if (strcmp(le_arg, "emit") == 0 || strcmp(le_arg, "xfce") == 0) {
 			emit_xfce(mpal.ra);
-		} else if (strcmp(*argv, "xterm") == 0) {
+		} else if (strcmp(le_arg, "xterm") == 0) {
 			emit_xterm(mpal.ra);
-		} else if (strcmp(*argv, "fg") == 0) {
+		} else if (strcmp(le_arg, "fg") == 0) {
 			xterm_fg = 1;
-		} else if (strcmp(*argv, "bg") == 0) {
+		} else if (strcmp(le_arg, "bg") == 0) {
 			xterm_bg = 1;
-		} else if (strcmp(*argv, "bd") == 0) {
+		} else if (strcmp(le_arg, "bd") == 0) {
 			xterm_bd = 1;
-		} else if (strcmp(*argv, "b0") == 0) {
+		} else if (strcmp(le_arg, "b0") == 0) {
 			mpal.la[0] = {0,0,0};
 			mpal.ra[0] = {0,0,0};
-		} else if (strcmp(*argv, "inv16") == 0) {
+		} else if (strcmp(le_arg, "inv16") == 0) {
 			decltype(mpal.ra) new_ra(mpal.ra.size());
 			for (size_t i = 0; i < mpal.ra.size(); ++i)
 				new_ra[i] = std::move(mpal.ra[~i % mpal.ra.size()]);
@@ -1081,38 +1115,38 @@ int main(int argc, char **argv)
 			 * h.l = 1 - 0.25 * h.s - h.l;
 			 * e = to_srgb888(to_srgb(h));
 			 */
-		} else if (strcmp(*argv, "ct256") == 0) {
+		} else if (strcmp(le_arg, "ct256") == 0) {
 			colortable_256();
 			colortable_16();
-		} else if (strcmp(*argv, "ct") == 0) {
+		} else if (strcmp(le_arg, "ct") == 0) {
 			colortable_16();
-		} else if (strcmp(*argv, "cxl") == 0) {
+		} else if (strcmp(le_arg, "cxl") == 0) {
 			cxl_command(mpal.la);
-		} else if (strcmp(*argv, "cxa") == 0) {
+		} else if (strcmp(le_arg, "cxa") == 0) {
 			cxa_command(mpal.ra);
-		} else if (strncmp(*argv, "cfgamma=", 8) == 0) {
+		} else if (strncmp(le_arg, "cfgamma=", 8) == 0) {
 			g_continuous_gamma = arg1;
-		} else if (strcmp(*argv, "loeq") == 0) {
+		} else if (strcmp(le_arg, "loeq") == 0) {
 			mpal.la = equalize(mpal.la, 9, 100 / 9.0, 100 * 8 / 9.0);
 			mod_la = true;
-		} else if (strncmp(*argv, "loeq=", 5) == 0) {
+		} else if (strncmp(le_arg, "loeq=", 5) == 0) {
 			char *end = nullptr;
-			arg1 = strtod(&argv[0][5], &end);
+			arg1 = strtod(&le_arg[5], &end);
 			double arg2 = *end == ',' ? strtod(end + 1, &end) : 100 / 9.0 * 8;
 			mpal.la = equalize(mpal.la, 9, arg1, arg2);
 			mod_la = true;
-		} else if (strcmp(*argv, "eq") == 0) {
+		} else if (strcmp(le_arg, "eq") == 0) {
 			mpal.la = equalize(mpal.la, 16, 100 / 16.0, 100);
 			mod_la = true;
-		} else if (strncmp(*argv, "eq=", 3) == 0) {
+		} else if (strncmp(le_arg, "eq=", 3) == 0) {
 			mpal.la = equalize(mpal.la, 16, arg1, 100);
 			mod_la = true;
-		} else if (strcmp(*argv, "syncfromrgb") == 0) {
+		} else if (strcmp(le_arg, "syncfromrgb") == 0) {
 			mpal.mod_ra();
-		} else if (strcmp(*argv, "syncfromlch") == 0) {
+		} else if (strcmp(le_arg, "syncfromlch") == 0) {
 			mpal.mod_la();
 		} else {
-			fprintf(stderr, "Unrecognized command: \"%s\"\n", *argv);
+			fprintf(stderr, "Unrecognized command: \"%s\"\n", le_arg);
 		}
 		if (mod_ra)
 			mpal.mod_ra();
