@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
-// SPDX-FileCopyrightText: 2019 Jan Engelhardt
+// SPDX-FileCopyrightText: 2019–2025 Jan Engelhardt
 /*
  *	I/O and glyph manipulation routines of the "VGA font assembler"
  */
@@ -10,6 +10,7 @@
 #include <memory>
 #include <numeric>
 #include <set>
+#include <span>
 #include <sstream>
 #include <string>
 #include <utility>
@@ -94,12 +95,42 @@ struct psf2_header {
 	uint32_t version, headersize, flags, length, charsize, height, width;
 };
 
+struct vertex {
+	bool operator<(const struct vertex &) const;
+	bool operator==(const struct vertex &) const;
+	bool operator!=(const struct vertex &a) const { return !operator==(a); }
+	int y, x;
+};
+
+struct edge {
+	bool operator<(const struct edge &) const;
+	bool operator==(const struct edge &) const;
+	bool operator!=(const struct edge &a) const { return !operator==(a); }
+	unsigned int trivial_dir() const;
+	struct vertex start_vtx, end_vtx;
+};
+
+template<typename T> class window : public std::span<T> {
+	public:
+	template<typename... A> window(size_t ofs, A &&...args) :
+		std::span<T>(std::forward<A>(args)...), m_ofs(ofs)
+	{}
+	T &operator[](size_t x) { return this->span::operator[]((x + m_ofs) % this->size()); }
+	const T &operator[](size_t x) const { return this->span::operator[]((x + m_ofs) % this->size()); }
+
+	private:
+	size_t m_ofs = 0;
+};
+
+using loose_edge_set = std::set<edge>;
+using closed_path = std::vector<edge>;
+
 class vectorizer final {
 	public:
 	vectorizer(const glyph &, int descent = 0);
-	std::vector<std::vector<edge>> simple();
-	std::vector<std::vector<edge>> n1();
-	std::vector<std::vector<edge>> n2(unsigned int flags = 0);
+	std::vector<closed_path> simple();
+	std::vector<closed_path> n1();
+	std::vector<closed_path> n2(unsigned int flags = 0);
 
 	/*
 	 * A distance of one pixel is mapped to this many vector font units.
@@ -116,15 +147,15 @@ class vectorizer final {
 
 	private:
 	void make_squares();
-	void internal_edge_delete();
-	unsigned int neigh_edges(unsigned int dir, const vertex &, std::set<edge>::iterator &, std::set<edge>::iterator &) const;
-	std::set<edge>::iterator next_edge(unsigned int dir, const edge &, unsigned int flags) const;
-	std::vector<edge> pop_poly(unsigned int flags);
-	void set(int, int);
+	void del_antipar();
+	unsigned int neigh_edges(unsigned int dir, const vertex &, loose_edge_set::iterator &, loose_edge_set::iterator &) const;
+	loose_edge_set::iterator next_edge(unsigned int dir, const edge &, unsigned int flags) const;
+	closed_path pop_poly(unsigned int flags);
+	void add_blot(int, int);
 
 	const glyph &m_glyph;
 	int m_descent = 0;
-	std::set<edge> emap;
+	loose_edge_set emap;
 	static const unsigned int P_SIMPLIFY_LINES = 1 << 0;
 };
 
@@ -658,18 +689,20 @@ static int load_pcf_props(FILE *fp, std::map<std::string, std::string> &map)
 	bool be = val & PCF_BYTE_MASK;
 	if (fread(&val, 4, 1, fp) != 1)
 		return -EINVAL;
-	auto numprop = be ? be32_to_cpu(val) : le32_to_cpu(val);
+	auto numprop = std::min(be ? be32_to_cpu(val) : le32_to_cpu(val), static_cast<uint32_t>(UINT32_MAX));
 	auto tbl_offset = ftell(fp);
-	fseek(fp, numprop * 9, SEEK_CUR);
-	fseek(fp, 4 - (ftell(fp) & 3), SEEK_CUR);
-	if (fread(&val, 4, 1, fp) != 1)
+	if (tbl_offset < 0)
 		return -EINVAL;
-	val = be ? be32_to_cpu(val) : le32_to_cpu(val);
+	if (fseek(fp, numprop * 9, SEEK_CUR) != 0 ||
+	    fseek(fp, 4 - (ftell(fp) & 3), SEEK_CUR) != 0 ||
+	    fread(&val, 4, 1, fp) != 1)
+		return -EINVAL;
+	val = std::min(UINT32_MAX, be ? be32_to_cpu(val) : le32_to_cpu(val));
 	std::string sblk;
 	sblk.resize(val);
-	if (fread(&sblk[0], sblk.size(), 1, fp) != 1)
+	if (fread(&sblk[0], sblk.size(), 1, fp) != 1 ||
+	    fseek(fp, tbl_offset, SEEK_SET) != 0)
 		return -EINVAL;
-	fseek(fp, tbl_offset, SEEK_SET);
 	for (unsigned int i = 0; i < numprop; ++i) {
 		if (fread(&val, 4, 1, fp) != 1)
 			return -EINVAL;
@@ -709,21 +742,23 @@ static int load_pcf_bitmaps(FILE *fp)
 	    fread(&bmpsize[0], 4, glypadopts, fp) != glypadopts)
 		return -EINVAL;
 	for (unsigned int i = 0; i < numbitmaps; ++i) {
-		offsets[i] = be ? be32_to_cpu(offsets[i]) : le32_to_cpu(offsets[i]);
+		offsets[i] = std::min(UINT32_MAX, be ? be32_to_cpu(offsets[i]) : le32_to_cpu(offsets[i]));
 		printf("bmp %u offset %u\n",
 			i, offsets[i]);
 	}
 	for (unsigned int i = 0; i < glypadopts; ++i) {
-		bmpsize[i] = be ? be32_to_cpu(bmpsize[i]) : le32_to_cpu(bmpsize[i]);
+		bmpsize[i] = std::min(UINT32_MAX, be ? be32_to_cpu(bmpsize[i]) : le32_to_cpu(bmpsize[i]));
 		printf("padopt %u size %u\n", i, bmpsize[i]);
 	}
 	std::string bmapbuf;
-	#define PCF_GLYPH_PAD_INDEX(f) ((f) & PCF_GLYPH_PAD_MASK)
+#define PCF_GLYPH_PAD_INDEX(f) ((f) & PCF_GLYPH_PAD_MASK)
+	printf("format %u\n", PCF_GLYPH_PAD_INDEX(fmt));
 	bmapbuf.resize(bmpsize[PCF_GLYPH_PAD_INDEX(fmt)]);
 	fprintf(stderr, "buf %zu\n", bmapbuf.size());
 	if (fread(&bmapbuf[0], bmapbuf.size(), 1, fp) != 1)
 		return -EINVAL;
 	return 0;
+#undef PCF_GLYPH_PAD_INDEX
 }
 
 int font::load_pcf(const char *filename)
@@ -778,9 +813,9 @@ static char32_t nextutf8(FILE *fp)
 {
 	unsigned int nbyte = 0;
 	auto ret = fgetc(fp);
-	if (ret == EOF || ret == 0xFF)
+	if (ret == EOF || ret < 0 || ret >= 0xFF)
 		return ~0U;
-	if (ret >= 0x00 && ret < 0xC0)
+	if (ret < 0xC0)
 		return ret;
 
 	if (ret >= 0xC0 && ret < 0xE0) nbyte = 2;
@@ -789,10 +824,10 @@ static char32_t nextutf8(FILE *fp)
 	else if (ret >= 0xF8 && ret < 0xFC) nbyte = 5;
 	else if (ret >= 0xFC && ret < 0xFE) nbyte = 6;
 
-	char32_t uc = ret & ~(~0U << (7 - nbyte));
+	uint32_t uc = ret & ~(~0U << (7 - nbyte));
 	for (unsigned int z = 1; z < nbyte; ++z) {
 		ret = fgetc(fp);
-		if (ret == EOF || ret == 0xFF || ((ret & 0xC0) != 0x80))
+		if (ret == EOF || ret < 0 || ret >= 0xFF || ((ret & 0xC0) != 0x80))
 			return ~0U;
 		uc <<= 6;
 		uc |= static_cast<unsigned char>(ret & 0x3F);
@@ -808,19 +843,24 @@ static char32_t nextucs2(FILE *fp)
 	auto y = fgetc(fp);
 	if (y == EOF)
 		return ~0U;
-	x |= y << 8;
+	x = x & 0xff;
+	x |= (y & 0xff) << 8;
 	return x < 0xffff ? x : ~0U;
 }
 
 static unsigned int psf_version(FILE *fp)
 {
-	uint8_t x = fgetc(fp), y = fgetc(fp);
+	int x = fgetc(fp), y = fgetc(fp);
+	if (x == EOF || y == EOF)
+		return 0;
 	if (x == PSF1_MAGIC0 && y == PSF1_MAGIC1)
 		return 1;
 	if (x != PSF2_MAGIC0 || y != PSF2_MAGIC1)
 		return 0;
 	x = fgetc(fp);
 	y = fgetc(fp);
+	if (x == EOF || y == EOF)
+		return 0;
 	return x == PSF2_MAGIC2 && y == PSF2_MAGIC3 ? 2 : 0;
 }
 
@@ -833,6 +873,7 @@ int font::load_psf(const char *file)
 	struct psf2_header hdr{};
 	switch (psf_version(fp.get())) {
 	case 0:
+		fprintf(stderr, "%s: unrecognized format\n", file);
 		return -EINVAL;
 	case 1: {
 		auto mode = fgetc(fp.get()), charsize = fgetc(fp.get());
@@ -858,12 +899,17 @@ int font::load_psf(const char *file)
 		hdr.flags      = le32_to_cpu(hdr.flags);
 		hdr.length     = le32_to_cpu(hdr.length);
 		hdr.charsize   = le32_to_cpu(hdr.charsize);
-		hdr.height     = le32_to_cpu(hdr.height);
+		hdr.height     = std::min(le32_to_cpu(hdr.height), static_cast<uint32_t>(UINT32_MAX));
 		hdr.width      = le32_to_cpu(hdr.width);
 		break;
 	}
 	}
 
+	if (hdr.charsize != (hdr.width + 7) / 8 * hdr.height) {
+		fprintf(stderr, "width=%u height=%u charsize=%u: awfully big PSF charsize!\n",
+			hdr.width, hdr.height, hdr.charsize);
+		return 0;
+	}
 	std::unique_ptr<char[]> buf(new char[hdr.charsize]);
 	size_t glyph_start = m_glyph.size();
 	for (size_t idx = 0; idx < hdr.length; ++idx) {
@@ -990,7 +1036,7 @@ int font::save_clt(const char *dir)
 int font::save_clt_glyph(const char *dir, size_t idx, char32_t codepoint)
 {
 	std::stringstream ss;
-	ss << dir << "/" << std::setfill('0') << std::setw(4) << std::hex << codepoint << ".txt";
+	ss << dir << "/" << std::setfill('0') << std::setw(4) << std::hex << static_cast<unsigned>(codepoint) << ".txt";
 	auto outpath = ss.str();
 	std::unique_ptr<FILE, deleter> fp(vfopen(outpath.c_str(), "w"));
 	if (fp == nullptr) {
@@ -999,7 +1045,7 @@ int font::save_clt_glyph(const char *dir, size_t idx, char32_t codepoint)
 	}
 	auto data = m_glyph[idx].as_pclt();
 	auto ret = fwrite(data.c_str(), data.size(), 1, fp.get());
-	if (ret < 0 || (data.size() > 0 && ret != 1)) {
+	if (data.size() > 0 && ret != 1) {
 		fprintf(stderr, "fwrite %s: %s\n", outpath.c_str(), strerror(-errno));
 		return -errno;
 	}
@@ -1057,7 +1103,7 @@ int font::save_pbm(const char *dir)
 int font::save_pbm_glyph(const char *dir, size_t idx, char32_t codepoint)
 {
 	std::stringstream ss;
-	ss << dir << "/" << std::setfill('0') << std::setw(4) << std::hex << codepoint << ".pbm";
+	ss << dir << "/" << std::setfill('0') << std::setw(4) << std::hex << static_cast<unsigned>(codepoint) << ".pbm";
 	auto outpath = ss.str();
 	std::unique_ptr<FILE, deleter> fp(::fopen(outpath.c_str(), "w"));
 	if (fp == nullptr) {
@@ -1066,7 +1112,7 @@ int font::save_pbm_glyph(const char *dir, size_t idx, char32_t codepoint)
 	}
 	auto data = m_glyph[idx].as_pbm();
 	auto ret = fwrite(data.c_str(), data.size(), 1, fp.get());
-	if (ret < 0 || (data.size() > 0 && ret != 1)) {
+	if (data.size() > 0 && ret != 1) {
 		fprintf(stderr, "fwrite %s: %s\n", outpath.c_str(), strerror(-errno));
 		return -errno;
 	}
@@ -1189,12 +1235,17 @@ int font::save_sfd(const char *file, enum vectoalg vt)
 	if (it != props.end()) {
 		char *end = nullptr;
 		auto a = strtoul(it->second.c_str(), &end, 0);
-		if (end == nullptr || end[0] != '/') {
-			fprintf(stderr, "What garbage is \"%s\"? Ignored -setprop request.\n", it->second.c_str());
-		} else if (end[0] == '/') {
-			auto b = strtoul(end + 1, nullptr, 0);
-			if (b == 0) {
-				fprintf(stderr, "What garbage is \"%s\"? Ignored -setprop request.\n", it->second.c_str());
+		if (end == nullptr) {
+			fprintf(stderr, "ssf \"%s\" ignored: something broke\n", it->second.c_str());
+		} else if (end == it->second.c_str() || *end != '/') {
+			fprintf(stderr, "ssf \"%s\" ignored: must adhere to form \"x/y\"\n", it->second.c_str());
+		} else {
+			auto txt = end + 1;
+			auto b = strtoul(txt, &end, 0);
+			if (end == nullptr || end == txt || *end != '\0') {
+				fprintf(stderr, "ssf \"%s\" ignored: must adhere to form \"x/y\"\n", it->second.c_str());
+			} else if (a == 0 || b == 0) {
+				fprintf(stderr, "ssf \"%s\" ignored: factors must not be 0\n", it->second.c_str());
 			} else {
 				m_ssfx = 2 * a;
 				m_ssfy = 2 * b;
@@ -1279,7 +1330,7 @@ vectorizer::vectorizer(const glyph &g, int desc) :
  * is implicit in the graph (emap) and a polygon is defined by the
  * smallest walk with right turns only.
  */
-void vectorizer::set(int x, int y)
+void vectorizer::add_blot(int x, int y)
 {
 	/* TTF/OTF spec: right side of line to be interior */
 	const int &sx = scale_factor_x, &sy = scale_factor_y;
@@ -1299,18 +1350,18 @@ void vectorizer::make_squares()
 		for (unsigned int x = 0; x < sz.w; ++x) {
 			bitpos ipos = y * sz.w + x;
 			if (m_glyph.m_data[ipos.byte] & ipos.mask)
-				set(x, yy);
+				add_blot(x, yy);
 		}
 	}
 }
 
-void vectorizer::internal_edge_delete()
+void vectorizer::del_antipar()
 {
 	/*
-	 * Remove overlaps: As enforced by set(), all the abstract polygons are
+	 * Remove overlaps: As enforced by add_blot(), all the abstract polygons are
 	 * added with the same orientation. Polygons at most touch, and never
 	 * overlap. Joining these abstract polygons simply requires removing
-	 * shared contradirectional edges. It follows by induction that the
+	 * antiparallel congruent edges. It follows by induction that the
 	 * intrinsic property {{smallest walk with right turns only} forms a
 	 * closed polygon} is kept.
 	 *
@@ -1342,7 +1393,7 @@ void vectorizer::internal_edge_delete()
  * Find the next edges (up to two) for @tail.
  */
 unsigned int vectorizer::neigh_edges(unsigned int cur_dir, const vertex &tail,
-    std::set<edge>::iterator &inward, std::set<edge>::iterator &outward) const
+    loose_edge_set::iterator &inward, loose_edge_set::iterator &outward) const
 {
 	inward = emap.lower_bound({tail, {INT_MIN, INT_MIN}});
 	if (inward == emap.end() || inward->start_vtx != tail) {
@@ -1359,11 +1410,11 @@ unsigned int vectorizer::neigh_edges(unsigned int cur_dir, const vertex &tail,
 	return 2;
 }
 
-std::set<edge>::iterator vectorizer::next_edge(unsigned int cur_dir,
+loose_edge_set::iterator vectorizer::next_edge(unsigned int cur_dir,
     const edge &cur_edge, unsigned int flags) const
 {
 	const auto &tail = cur_edge.end_vtx;
-	std::set<edge>::iterator inward, outward;
+	loose_edge_set::iterator inward, outward;
 	auto ret = neigh_edges(cur_dir, tail, inward, outward);
 	if (!(flags & P_ISTHMUS) || ret <= 1)
 		return inward;
@@ -1398,6 +1449,9 @@ std::set<edge>::iterator vectorizer::next_edge(unsigned int cur_dir,
 	 *   ....####
 	 *
 	 * Right now, we are only testing for A1+A2. Might be enough...?
+	 *
+	 * Not good enough. If we do not consistently return `outward`, this
+	 * routine produces funny patterns with cp437 point 0xb1 (U+2592).
 	 */
 	vertex bmp;
 	if (cur_dir == 0)
@@ -1437,7 +1491,6 @@ std::set<edge>::iterator vectorizer::next_edge(unsigned int cur_dir,
 		return inward;
 	if (cur_dir == 270 && testbit_c(m_glyph, bmp.x - 2, bmp.y + 1) && testbit_c(m_glyph, bmp.x - 1, bmp.y + 2))
 		return inward;
-
 	return outward;
 }
 
@@ -1446,26 +1499,24 @@ std::set<edge>::iterator vectorizer::next_edge(unsigned int cur_dir,
  *
  * The vectorizer class only keeps a loose set of edge descriptions, but all
  * these edges form valid closed polygons (cf. vectorizer::set,
- * vectorizer::internal_edge_removal). Thus, by starting a walk at an arbitrary
+ * vectorizer::del_antipar). Thus, by starting a walk at an arbitrary
  * edge and following the path with "right turns only" until we see the same
  * edge again, that will be our polygon.
  */
-std::vector<edge> vectorizer::pop_poly(unsigned int flags)
+closed_path vectorizer::pop_poly(unsigned int flags)
 {
-	std::vector<edge> poly;
-	if (emap.size() == 0)
+	closed_path poly;
+	if (emap.empty())
 		return poly;
 	poly.push_back(*emap.begin());
 	emap.erase(emap.begin());
 	auto prev_dir = poly[0].trivial_dir();
 
-	while (true) {
-		if (emap.size() == 0)
+	while (emap.size() > 0) {
+		auto &tail_vtx = poly.back().end_vtx;
+		if (tail_vtx == poly.front().start_vtx)
 			break;
-		auto &tail_vtx = poly.rbegin()->end_vtx;
-		if (tail_vtx == poly.cbegin()->start_vtx)
-			break;
-		auto next = next_edge(prev_dir, *poly.rbegin(), flags);
+		auto next = next_edge(prev_dir, poly.back(), flags);
 		if (next == emap.cend()) {
 			fprintf(stderr, "unclosed poly wtf?!\n");
 			break;
@@ -1488,11 +1539,11 @@ std::vector<edge> vectorizer::pop_poly(unsigned int flags)
 	return poly;
 }
 
-std::vector<std::vector<edge>> vectorizer::simple()
+std::vector<closed_path> vectorizer::simple()
 {
 	make_squares();
-	internal_edge_delete();
-	std::vector<std::vector<edge>> pmap;
+	del_antipar();
+	std::vector<closed_path> pmap;
 	while (true) {
 		auto poly = pop_poly(P_SIMPLIFY_LINES);
 		if (poly.size() == 0)
@@ -1502,13 +1553,13 @@ std::vector<std::vector<edge>> vectorizer::simple()
 	return pmap;
 }
 
-std::vector<std::vector<edge>> vectorizer::n1()
+std::vector<closed_path> vectorizer::n1()
 {
 	auto &g = m_glyph;
 	const auto &sz = g.m_size;
-	for (unsigned int uy = 0; uy < sz.h; ++uy) {
-		int y = sz.h - 1 - static_cast<int>(uy) - m_descent;
-		for (unsigned int ux = 0; ux < sz.w; ++ux) {
+	for (int uy = 0; uy < static_cast<int>(sz.h); ++uy) {
+		int y = sz.h - 1 - uy - m_descent;
+		for (int ux = 0; ux < static_cast<int>(sz.w); ++ux) {
 			bitpos ipos = uy * sz.w + ux;
 			int x = ux;
 
@@ -1557,8 +1608,8 @@ std::vector<std::vector<edge>> vectorizer::n1()
 		}
 	}
 
-	internal_edge_delete();
-	std::vector<std::vector<edge>> pmap;
+	del_antipar();
+	std::vector<closed_path> pmap;
 	while (true) {
 		auto poly = pop_poly(P_SIMPLIFY_LINES);
 		if (poly.size() == 0)
@@ -1568,89 +1619,85 @@ std::vector<std::vector<edge>> vectorizer::n1()
 	return pmap;
 }
 
-static void n2_angle(std::vector<edge> &poly, unsigned int sx, unsigned int sy)
+static void n2_angle(closed_path &poly, unsigned int sx, unsigned int sy)
 {
 	static const unsigned int M_HEAD = 0x20, M_TAIL = 0x02,
 		M_XHEAD = 0x10, M_XTAIL = 0x01;
+	std::vector<unsigned int> tdir(poly.size());
 	std::vector<unsigned int> flags(poly.size());
 
+	for (size_t i = 0; i < poly.size(); ++i)
+		tdir[i] = poly[i].trivial_dir();
+
 	/*
-	 * It's a closed polygon and so it does not matter which edge
-	 * processing starts at. (xm3 = x minus 3, x00 = current edge, xp3 = x
-	 * plus 3, etc.)
+	 * A sliding window of 7 edges (conceptually, 3 to the left, current
+	 * edge, and 3 to right) is used. Since the input is a cyclic path, it
+	 * does not matter at which edge the processing starts at.
 	 *
-	 * In the loop, edges are marked with bitflags M_HEAD/M_TAIL to
-	 * indicate that a particular edge allows modification of the start or
-	 * end vertex (or both).
+	 * Edges are marked with bitflags M_HEAD/M_TAIL to indicate that a
+	 * particular edge qualifies for modification of the start or end
+	 * vertex (or both).
 	 *
 	 * M_XHEAD/M_XTAIL are used as veto flags. (We cannot just use e.g.
 	 * `flags[xm3] & ~M_XHEAD` in one iteration, because a subsequent
 	 * iteration may set it again via e.g. `flags[xm2] |= M_HEAD`.)
 	 */
-	for (size_t xm3 = 0; xm3 < poly.size(); ++xm3) {
-		auto xm2 = (xm3 + 1) % poly.size();
-		auto xm1 = (xm3 + 2) % poly.size();
-		auto x00 = (xm3 + 3) % poly.size();
-		auto xp1 = (xm3 + 4) % poly.size();
-		auto xp2 = (xm3 + 5) % poly.size();
-		auto xp3 = (xm3 + 6) % poly.size();
-		auto dm3 = poly[xm3].trivial_dir(), dm2 = poly[xm2].trivial_dir();
-		auto dm1 = poly[xm1].trivial_dir(), d00 = poly[x00].trivial_dir();
-		auto dp1 = poly[xp1].trivial_dir(), dp2 = poly[xp2].trivial_dir();
-		auto dp3 = poly[xp3].trivial_dir();
+	for (size_t edge_idx = 0; edge_idx < poly.size(); ++edge_idx) {
+		window<unsigned int> D(edge_idx, tdir);
+		window<unsigned int> f(edge_idx, flags);
 
 #if 0
-		printf("I%zu dm3:\e[32m%d\e[0m,dm2:\e[32m%d\e[0m,"
-			"dm1:\e[32m%d\e[0m,d:\e[32m%d\e[0m,dp1:\e[32m%d\e[0m,"
-			"dp2:\e[32m%d\e[0m,dp3:\e[32m%d\e[0m\n",
-			x00, dm3, dm2, dm1, d00, dp1, dp2, dp3);
+		printf("I%zu [-3]:\e[32m%d\e[0m,[-2]:\e[32m%d\e[0m,"
+			"[-1]:\e[32m%d\e[0m,[0]:\e[32m%d\e[0m,[1]:\e[32m%d\e[0m,"
+			"[2]:\e[32m%d\e[0m,[3]:\e[32m%d\e[0m\n",
+			x00, D[-3], D[-2], D[-1], D[0], D[1], D[2], D[3]);
 #endif
 
-		if (d00 == dm2 && d00 == dp2) {
+		if (D[0] == D[-2] && D[0] == D[2]) {
 			/* _|~|_ or ~|_|~ pattern seen */
-			if ((dm3 == d00 || dm3 == dp1) &&
-			    (dp3 == d00 || dp3 == dm1) &&
-			    dm1 == (dm2 + 270) % 360 && dp1 == (dm2 + 90) % 360) {
+			if ((D[-3] == D[0] || D[-3] == D[1]) &&
+			    (D[3] == D[0] || D[3] == D[-1]) &&
+			    D[-1] == (D[-2] + 270) % 360 && D[1] == (D[-2] + 90) % 360) {
 				/* pimple __|~|__ ('f', '4'), retain */
-				flags[xm2] |= M_XTAIL;
-				flags[xm1]  = M_XHEAD | M_XTAIL;
-				flags[x00]  = M_XHEAD | M_XTAIL;
-				flags[xp1]  = M_XHEAD | M_XTAIL;
-				flags[xp2] |= M_XHEAD;
+				f[-2] |= M_XTAIL;
+				f[-1]  = M_XHEAD | M_XTAIL;
+				f[0]   = M_XHEAD | M_XTAIL;
+				f[1]   = M_XHEAD | M_XTAIL;
+				f[2]  |= M_XHEAD;
 				continue;
 			}
 
-			if (dm1 == (dm2 + 90) % 360 && dp1 == (dm2 + 270) % 360) {
+			if (D[-1] == (D[-2] + 90) % 360 && D[1] == (D[-2] + 270) % 360) {
 				/* dimple ~~|_|~~ ('8'), sink it */
-				if (dm3 == dm2) {
+				if (D[-3] == D[-2]) {
 					/* with left-side flat zone */
-					flags[xm2] |= M_TAIL;
-					flags[xm1]  = M_HEAD | M_TAIL;
-					flags[x00] |= M_HEAD;
+					f[-2] |= M_TAIL;
+					f[-1]  = M_HEAD | M_TAIL;
+					f[0]  |= M_HEAD;
 				}
-				if (dp3 == dp2) {
+				if (D[3] == D[2]) {
 					/* with right-side flat zone */
-					flags[x00] |= M_TAIL;
-					flags[xp1]  = M_HEAD | M_TAIL;
-					flags[xp2] |= M_HEAD;
+					f[0] |= M_TAIL;
+					f[1]  = M_HEAD | M_TAIL;
+					f[2] |= M_HEAD;
 				}
 				continue;
 			}
 		}
 
 		/* Test for chicane */
-		if (dm1 != dp1)
+		if (D[-1] != D[1])
 			continue;
-		if ((d00 + 270) % 360 != dp1 && (d00 + 90) % 360 != dp1)
+		if ((D[0] + 270) % 360 != D[1] && (D[0] + 90) % 360 != D[1])
 			continue;
 
 		/* #5: serif (ramp), topleft of ibmvga916 'E' */
-		if (dm2 == dm1 && d00 == (dm1 + 270) % 360 &&
-		    dp1 == dm1 && dp2 == (dm1 + 90) % 360 && dp3 == dp2)
+		if (D[-2] == D[-1] && D[0] == (D[-1] + 270) % 360 &&
+		    D[1] == D[-1] && D[2] == (D[-1] + 90) % 360 && D[3] == D[2])
 			continue;
 		/* bottomleft of ibmvga916 'E' */
-		if (dm3 == dm2 && dm1 == (dm2 + 90) % 360 &&
-		    d00 == (dm2 + 180) % 360 && dp1 == dm1 && dp2 == dp1)
+		if (D[-3] == D[-2] && D[-1] == (D[-2] + 90) % 360 &&
+		    D[0] == (D[-2] + 180) % 360 && D[1] == D[-1] && D[2] == D[1])
 			continue;
 
 		/*
@@ -1659,17 +1706,17 @@ static void n2_angle(std::vector<edge> &poly, unsigned int sx, unsigned int sy)
 		 * #3: stairs midpart,
 		 * #4: top of stairs (implies no sump)
 		 */
-		flags[xm1] |= M_TAIL;
-		flags[x00]  = M_HEAD | M_TAIL;
-		flags[xp1] |= M_HEAD;
+		f[-1] |= M_TAIL;
+		f[0]   = M_HEAD | M_TAIL;
+		f[1]  |= M_HEAD;
 
-		if (dp2 == d00) {
-			flags[xp1] |= M_TAIL;
-			flags[xp2] |= M_HEAD;
+		if (D[2] == D[0]) {
+			f[1] |= M_TAIL;
+			f[2] |= M_HEAD;
 		}
-		if (dm2 == d00) {
-			flags[xm2] |= M_TAIL;
-			flags[xm1] |= M_HEAD;
+		if (D[-2] == D[0]) {
+			f[-2] |= M_TAIL;
+			f[-1] |= M_HEAD;
 		}
 	}
 
@@ -1730,12 +1777,12 @@ static void n2_angle(std::vector<edge> &poly, unsigned int sx, unsigned int sy)
 	}
 }
 
-std::vector<std::vector<edge>> vectorizer::n2(unsigned int flags)
+std::vector<closed_path> vectorizer::n2(unsigned int flags)
 {
 	flags &= P_ISTHMUS;
 	make_squares();
-	internal_edge_delete();
-	std::vector<std::vector<edge>> pmap;
+	del_antipar();
+	std::vector<closed_path> pmap;
 	while (true) {
 		/* Have all edges retain length 1 */
 		auto poly = pop_poly(flags);
@@ -1762,7 +1809,7 @@ void font::save_sfd_glyph(FILE *fp, size_t idx, char32_t cp, int asc, int desc,
 	fprintf(fp, "Fore\n");
 	fprintf(fp, "SplineSet\n");
 
-	std::vector<std::vector<edge>> pmap;
+	std::vector<closed_path> pmap;
 	vectorizer vct(m_glyph[idx], desc);
 	vct.scale_factor_x = m_ssfx;
 	vct.scale_factor_y = m_ssfy;
@@ -1775,7 +1822,7 @@ void font::save_sfd_glyph(FILE *fp, size_t idx, char32_t cp, int asc, int desc,
 	else if (vt == V_N2EV)
 		pmap = vct.n2(vectorizer::P_ISTHMUS);
 	for (const auto &poly : pmap) {
-		const auto &v1 = poly.cbegin()->start_vtx;
+		const auto &v1 = poly.front().start_vtx;
 		fprintf(fp, "%d %d m 25\n", v1.x, v1.y);
 		for (const auto &edge : poly)
 			fprintf(fp, " %d %d l 25\n", edge.end_vtx.x, edge.end_vtx.y);
