@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
-// SPDX-FileCopyrightText: 2019 Jan Engelhardt
+// SPDX-FileCopyrightText: 2019–2025 Jan Engelhardt
 /*
  *	Command-line interface of the "VGA font assembler"
  */
@@ -8,11 +8,11 @@
 #include <memory>
 #include <string>
 #include <utility>
+#include <cstdint>
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
 #include <fcntl.h>
-#include <getopt.h>
 #include <unistd.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
@@ -391,8 +391,8 @@ static bool vf_upscale(font &f, char **args)
 {
 	auto xf = strtol(args[0], nullptr, 0);
 	auto yf = strtol(args[1], nullptr, 0);
-	if (xf <= 0 || xf <= 0) {
-		fprintf(stderr, "Error: scaling factor(s) should be positive and not zero.\n");
+	if (xf <= 0 || yf <= 0) {
+		fprintf(stderr, "Error: scaling factor(s) should be positive and greater than zero.\n");
 		return false;
 	}
 	f.upscale(vfsize(xf, yf));
@@ -410,7 +410,7 @@ static void vf_extract_sfh(const char *sfhblk, unsigned int num_fonts,
 	for (unsigned int i = 0; i < num_fonts; ++i) {
 		struct cpi_screenfont_header sfh;
 		memcpy(&sfh, sfhblk, sizeof(sfh));
-		sfh.num_chars = le16_to_cpu(sfh.num_chars);
+		sfh.num_chars = std::min(le16_to_cpu(sfh.num_chars), static_cast<uint16_t>(UINT16_MAX));
 		printf("SFH: %ux%u pixels x %u chars\n", sfh.width, sfh.height,
 		       sfh.num_chars);
 		if (sfh.width == 0 || sfh.height == 0 || sfh.num_chars == 0)
@@ -470,7 +470,7 @@ static int vf_extract_cpi2(const char *vdata, size_t vsize,
 		return -EINVAL;
 
 	memcpy(&fih, vdata + ffh.fih_offset, sizeof(fih));
-	fih.num_codepages = le16_to_cpu(fih.num_codepages);
+	fih.num_codepages = std::min(le16_to_cpu(fih.num_codepages), static_cast<uint16_t>(UINT16_MAX));
 
 	const char *cpeblk = vdata + ffh.fih_offset + sizeof(fih);
 	for (unsigned int i = 0; i < fih.num_codepages; ++i) {
@@ -505,7 +505,7 @@ static int vf_extract_cpi2(const char *vdata, size_t vsize,
 			return -EINVAL;
 		memcpy(&cpih, vdata + cpeh.cpih_offset, sizeof(cpih));
 		cpih.version   = le16_to_cpu(cpih.version);
-		cpih.num_fonts = le16_to_cpu(cpih.num_fonts);
+		cpih.num_fonts = std::min(le16_to_cpu(cpih.num_fonts), static_cast<uint16_t>(UINT16_MAX));
 		cpih.size      = le16_to_cpu(cpih.size);
 		printf("CPIH: version=%u fonts=%u size=%u\n", cpih.version,
 		       cpih.num_fonts, cpih.size);
@@ -622,13 +622,11 @@ static const struct vf_command {
 
 int main(int argc, char **argv)
 {
-	--argc;
-	++argv;
-	if (argc == 0) {
-		fprintf(stderr, "You should specify some commlist.\n");
-		return EXIT_FAILURE;
-	}
 	font f;
+	if (argc > 0) {
+		--argc;
+		++argv;
+	}
 	while (argc > 0) {
 		if (argv[0][0] == '-')
 			++argv[0];
